@@ -1,15 +1,18 @@
+# coding=utf-8
+
 """
 Top level imports in the interpolation module.
 """
 
 __author__ = "Morten Lind"
-__copyright__ = "Morten Lind 2009-2012"
+__copyright__ = "Morten Lind 2009-2015"
 __credits__ = ["Morten Lind"]
 __license__ = "GPLv3"
 __maintainer__ = "Morten Lind"
-__email__ = "morten@lind.no-ip.org"
+__email__ = "morten@lind.dyndns.dk"
 __status__ = "Production"
 
+
 from .r3interpolation import R3Interpolation
 from .se3interpolation import SE3Interpolation
 from .so3interpolation import SO3Interpolation
